import { declare } from "@babel/helper-plugin-utils";
import { types as t, template } from "@babel/core";
const TRACE_ID = "__source";
const FILE_NAME_VAR = "_jsxFileName";
const createNodeFromNullish = (val, fn) => val == null ? t.nullLiteral() : fn(val);
export default declare(api => {
  api.assertVersion(7);
  function makeTrace(fileNameIdentifier, {
    line,
    column
  }) {
    const fileLineLiteral = createNodeFromNullish(line, t.numericLiteral);
    const fileColumnLiteral = createNodeFromNullish(column, c => t.numericLiteral(c + 1));
    return template.expression.ast`{
      fileName: ${fileNameIdentifier},
      lineNumber: ${fileLineLiteral},
      columnNumber: ${fileColumnLiteral},
    }`;
  }
  const isSourceAttr = attr => t.isJSXAttribute(attr) && attr.name.name === TRACE_ID;
  return {
    name: "transform-react-jsx-source",
    visitor: {
      JSXOpeningElement(path, state) {
        const {
          node
        } = path;
        if (!node.loc || path.node.attributes.some(isSourceAttr)) {
          return;
        }
        if (!state.fileNameIdentifier) {
          const fileNameId = path.scope.generateUidIdentifier(FILE_NAME_VAR);
          state.fileNameIdentifier = fileNameId;
          path.scope.getProgramParent().push({
            id: fileNameId,
            init: t.stringLiteral(state.filename || "")
          });
        }
        node.attributes.push(t.jsxAttribute(t.jsxIdentifier(TRACE_ID), t.jsxExpressionContainer(makeTrace(t.cloneNode(state.fileNameIdentifier), node.loc.start))));
      }
    }
  };
});

//# sourceMappingURL=index.js.map
